/*
 * Copyright (c) 2016, 2016, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package com.oracle.truffle.r.library.methods;

import com.oracle.truffle.api.CompilerDirectives.TruffleBoundary;
import com.oracle.truffle.api.dsl.Fallback;
import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.api.nodes.Node;
import com.oracle.truffle.r.nodes.RASTUtils;
import com.oracle.truffle.r.nodes.builtin.RExternalBuiltinNode;
import com.oracle.truffle.r.nodes.builtin.RList2EnvNode;
import com.oracle.truffle.r.nodes.builtin.RList2EnvNodeGen;
import com.oracle.truffle.r.nodes.runtime.RASTDeparse;
import com.oracle.truffle.r.runtime.RError;
import com.oracle.truffle.r.runtime.data.RDataFactory;
import com.oracle.truffle.r.runtime.data.RLanguage;
import com.oracle.truffle.r.runtime.data.RList;
import com.oracle.truffle.r.runtime.env.REnvironment;
import com.oracle.truffle.r.runtime.nodes.RSyntaxNode;

public abstract class SubstituteDirect extends RExternalBuiltinNode.Arg2 {
    @Child private RList2EnvNode list2EnvNode;

    @Specialization
    @TruffleBoundary
    protected Object substituteDirect(Object object, REnvironment env) {
        if (object instanceof RLanguage) {
            RLanguage lang = (RLanguage) object;
            RSyntaxNode snode = lang.getRep().asRSyntaxNode();
            RSyntaxNode subRNode = snode.substituteImpl(env);
            // remove old source sections
            clearSourceSection(subRNode.asRNode());
            // create source for entire tree
            RASTDeparse.ensureSourceSection(subRNode);
            return RASTUtils.createLanguageElement(subRNode.asRNode());
        } else {
            return object;
        }
    }

    @Specialization
    @TruffleBoundary
    protected Object substituteDirect(Object object, RList list) {
        REnvironment env = RDataFactory.createNewEnv(null);
        env.setParent(REnvironment.baseEnv());
        list2Env(list, env);
        return substituteDirect(object, env);
    }

    @SuppressWarnings("unused")
    @Fallback
    protected Object substituteDirect(Object object, Object env) {
        throw RError.error(this, RError.Message.INVALID_OR_UNIMPLEMENTED_ARGUMENTS);
    }

    private REnvironment list2Env(RList list, REnvironment env) {
        if (list2EnvNode == null) {
            list2EnvNode = insert(RList2EnvNodeGen.create());
        }
        return list2EnvNode.execute(list, env);
    }

    private static void clearSourceSection(Node node) {
        node.accept(n -> {
            n.clearSourceSection();
            return true;
        });
    }

}
